/* ***** BEGIN LICENSE BLOCK *****
    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License Version
    1.1 (the "License"); you may not use this file except in compliance with
    the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is Quick Locale Switcher code.

    The Initial Developer of the Original Code is Martijn Kooij a.k.a. Captain Caveman.

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead
    of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK ***** */
"use strict";
/* jshint ignore:start */
if (typeof Cc == "undefined") var Cc = Components.classes;
if (typeof Ci == "undefined") var Ci = Components.interfaces;
if (typeof Cu == "undefined") var Cu = Components.utils;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("chrome://qls/content/qlscommon.jsm");
/* jshint ignore:end */

var QLSOverlay = {

  //holdover from old fox?
  QLSPreInit: function()
  {
    if (!commonQLS.oQLSPref && !this.bQLSInitializing) {
      this.bQLSInitializing = true;
      setTimeout(function() {
        QLSOverlay.QLSInit();
      }, 500);
    }
  },

  QLSInit: function()
  {
    this.QLSPrefObserver = {
      observe: function(subject, topic, prefName) {
        if (topic == "nsPref:changed" && prefName == "general.useragent.locale")
          QLSOverlay.CheckLocale();

        if (topic == "nsPref:changed" && prefName == "intl.accept_languages")
          //commonQLS.GetAcceptLanguages(false);
          commonQLS.GetAcceptLanguages(true);

        if (topic == "nsPref:changed" &&
            prefName == "spellchecker.dictionary" &&
            commonQLS.GetQLSPrefBool("switch_contentlocale") &&
            commonQLS.GetCCode(commonQLS.GetQLSPref("contentlocale")) != commonQLS.GetSpellCheckerDictionary() &&
            commonQLS.GetQLSPref("contentlocale") != commonQLS.GetSpellCheckerDictionary()) {
          commonQLS.SetQLSPref("previouscontentlocale", commonQLS.GetQLSPref("contentlocale"));
          commonQLS.SetQLSPref("contentlocale", commonQLS.GetSpellCheckerDictionary());
          QLSOverlay.CheckLocale();
        }

      }
    };

    this.QLSSettingsObserver = {
      observe: function(subject, topic, state) {
        if (topic == "qls-settings" && state == 'OK')
          QLSOverlay.ApplySettings();
      }
    };

    this.QLSHTTPRequestObserver = {
      //observe: function(subject, topic, state) {
      observe: function(subject, topic) {
        if (topic == "http-on-modify-request") {
          var oHTTPChannel = subject.QueryInterface(Ci.nsIHttpChannel),
          sURL = oHTTPChannel.URI.spec;

          var oBrowser = QLSOverlay.QLSGetBrowserFromChannel(oHTTPChannel);
          if (oBrowser)
            QLSOverlay.QLSModifyAcceptLanguageHeader(oHTTPChannel, sURL);
        }
      }
    };

    Services.prefs.addObserver("general.useragent.locale", this.QLSPrefObserver, false);
    Services.prefs.addObserver("intl.accept_languages", this.QLSPrefObserver, false);
    Services.prefs.addObserver("spellchecker.dictionary", this.QLSPrefObserver, false);

    Services.obs.addObserver(this.QLSSettingsObserver, "qls-settings", false);
    Services.obs.addObserver(this.QLSHTTPRequestObserver, "http-on-modify-request", false);

    var sDefaultLocale = commonQLS.GetQLSPref("locale");
    if (!sDefaultLocale || sDefaultLocale == "")
      sDefaultLocale = commonQLS.GetLocale();

    if (!commonQLS.oQLSPref.prefHasUserValue('locale'))
      commonQLS.SetQLSPref("locale", sDefaultLocale);

    if (!commonQLS.oQLSPref.prefHasUserValue('fallbacklocale'))
      commonQLS.SetQLSPref("fallbacklocale", sDefaultLocale);

    if (!commonQLS.oQLSPref.prefHasUserValue('contentlocale'))
      commonQLS.SetQLSPref("contentlocale", sDefaultLocale);

    if (!commonQLS.oQLSPref.prefHasUserValue('previouslocale'))
      commonQLS.SetQLSPref("previouslocale", sDefaultLocale);

    if (!commonQLS.oQLSPref.prefHasUserValue('previouscontentlocale'))
      commonQLS.SetQLSPref("previouscontentlocale", sDefaultLocale);

    commonQLS.sDefaultLocale = commonQLS.GetDefaultLocale();
    if (!commonQLS.sDefaultLocale || commonQLS.sDefaultLocale == "")
      commonQLS.sDefaultLocale = "en-US";
    if (commonQLS.sDefaultLocale.indexOf("-") == -1)
      commonQLS.sDefaultLocale = QLSOverlay.CCodeToLocale(commonQLS.sDefaultLocale);

    var oKey = document.getElementById("qls-key-switchback");
    oKey.setAttribute("modifiers", commonQLS.GetQLSPref("switchback.modifiers"));
    oKey.setAttribute("key", commonQLS.GetQLSPref("switchback.key"));

    var oBrowser = document.getElementById('content');
    if (oBrowser) {
      oBrowser.addEventListener("DOMContentLoaded", function load() {
        oBrowser.removeEventListener("load",load,false);
        if (!QLSOverlay.preventAutoSwitchOnDOMContentLoaded) {
          QLSOverlay.preventAutoSwitchOnDOMContentLoaded = true;
          QLSOverlay.QLSCheckContentLocale(false);
        }
      },true);
      var oTabs = oBrowser.tabContainer;
      if (!oTabs) {
        oTabs = oBrowser.mPanelContainer;
        oTabs.addEventListener("select", function() {
          QLSOverlay.QLSCheckContentLocale(true);
        },false);
      } else {
        oTabs.addEventListener("TabSelect", function() {
          QLSOverlay.QLSCheckContentLocale(true);
        },false);
      }
    }

    /*Gecko 2.0>, customize toolbar done...*/
    window.addEventListener("aftercustomization", function load() {
      window.removeEventListener("load",load,false);
      QLSOverlay.ApplySettings();
    }, false);

    QLSOverlay.ApplySettings();
    QLSOverlay.InitDefault();
    QLSOverlay.CheckLocale();
    commonQLS.QLSSwitchingAcceptLanguages = false;

    var oContent = document.getElementById('content-frame');
    if (!oContent)
      return;
    document.addEventListener("load", function load() {
      document.removeEventListener("load",load,false);
      QLSOverlay.QLSTBCheckAddressField();
      oContent.addEventListener("focus", QLSOverlay.QLSTBCheckAddressField, true);
    },true);
  },

  QLSGetBrowserFromChannel: function(aChannel)
  {
    try {
      var notificationCallbacks = aChannel.notificationCallbacks ? aChannel.notificationCallbacks : aChannel.loadGroup.notificationCallbacks;
      if (!notificationCallbacks) return null;

      var domWin = notificationCallbacks.getInterface(Ci.nsIDOMWindow);
      if (!domWin)
        return null;
      if (domWin.top.document.location != domWin.document.location)
        return null;

      return gBrowser.getBrowserForDocument(domWin.top.document);
    } catch (e) {
      return null;
    }
  },

  QLSDeInit: function()
  {
    try {
      Services.obs.removeObserver(this.QLSSettingsObserver, "qls-settings");
      Services.obs.removeObserver(this.QLSHTTPRequestObserver, "http-on-modify-request");
      Services.prefs.removeObserver("general.useragent.locale", this.QLSPrefObserver);
      Services.prefs.removeObserver("intl.accept_languages", this.QLSPrefObserver);
      Services.prefs.removeObserver("spellchecker.dictionary", this.QLSPrefObserver);

      this.QLSPrefObserver = null;
      this.QLSSettingsObserver = null;
    } catch (e) {}
  },

  InitDefault: function()
  {
    //seamonkey
    if (document.getElementById('taskPopup'))
      commonQLS.InitDefaultSeamonkey();
    //rename default menuitem
    var el = document.getElementById("defaultQLS");
    if (el) {
      var sLabel = el.getAttribute('statustext') + " (" + commonQLS.sDefaultLocale + ")";
      el.setAttribute("label", sLabel);
      el.setAttribute("ccode", commonQLS.sDefaultLocale);
    }
  },

  ApplySettings: function()
  {
    var oMItem,
    oSItem,
    el,
    i,
    bShowSep = false,
    aAllMItems = commonQLS.QLSTLDToLocaleP().split(","),
    //aAllMItems = commonQLS.QLS_ALL_LOCALES.split("#"),
    aMItems = commonQLS.GetQLSPref("visiblemenuitems").split("#");

    function setAtr(el,sLocale,sLabel)
    {
      el = document.getElementById(el);
      if (!el)
        return;
      sLocale = commonQLS.GetQLSPref(sLocale);
      sLabel = commonQLS.GetQLSPref(sLabel);
      if (sLocale == "" || sLabel == "")
        return;

      bShowSep = true;
      el.setAttribute("hidden", false);
      el.setAttribute("clocale", sLocale);
      el.setAttribute("ccode", sLocale.toLowerCase());
      el.setAttribute("label", sLabel);
    }
    //custom lang menuitems
    setAtr("qlscustom1","customlocale1","customlabel1",1);
    setAtr("qlscustom2","customlocale2","customlabel2",1);
    setAtr("qlscustom3","customlocale3","customlabel3",1);
    if (bShowSep == true) {
      el = document.getElementById("msCavemanQLS2");
      if (el)
        el.setAttribute("hidden", false);
    }

    //show checked? menuitems
    for (i = 0; i < aAllMItems.length; i++) {
      el = document.getElementById(aAllMItems[i]);
      if (el) {
        if (el.getAttribute("ui_checked") == 'true' || el.getAttribute("content_checked") == 'true')
          el.setAttribute("hidden",false);
        else
          el.setAttribute("hidden",true);
      }
    }
    //show visible? menuitems
    for (i = aMItems.length - 1; i > -1; i--) {
      el = document.getElementById(aMItems[i]);
      if (el) {
        var oParent = el.parentNode;
        oParent.removeChild(el);
        el.setAttribute("hidden",false);
        oParent.insertBefore(el,oParent.childNodes[6]);
      }
    }

    oSItem = document.getElementById("statusbar-QLS");
    oMItem = document.getElementById("mCavemanQLS");
    if (commonQLS.GetQLSPrefBool("hide_statusbar") && oSItem)
      oSItem.setAttribute("hidden", true);
    else if (oSItem)
      oSItem.setAttribute("hidden", false);
    if (commonQLS.GetQLSPrefBool("hide_tools") && oMItem)
      oMItem.setAttribute("hidden", true);
    else if (oMItem)
      oMItem.setAttribute("hidden", false);

    QLSOverlay.QLSInitHostLocales(true);
    window.setTimeout(function() {
      QLSOverlay.QLSTBCheckAddressField();
    }, 500);
    QLSOverlay.QLSInitStatusBar(commonQLS.GetQLSPref("locale"), commonQLS.GetQLSPref("contentlocale"));
    QLSOverlay.QLSCheckContentLocale(false);
  },

  CheckLocale: function()
  {
    //Uncheck all.
    QLSOverlay.UnCheckLocale();

    var oElements,
    i,
    //Check ui locale
    sLocale = commonQLS.GetQLSPref("locale"),
    oElement = document.getElementById(sLocale);

    if (!oElement) {
      oElements = document.getElementsByAttribute('ccode', sLocale.toLowerCase());
      for (i = 0; i < oElements.length; i++) {
        if (oElements[i].nodeName == "menuitem")
          oElement = oElements[i];
      }
    }

    if (!oElement) {
      sLocale = commonQLS.GetSeamonkeyLocale();
      if (sLocale && sLocale != "")
        oElement = document.getElementById(sLocale);
      if (!oElement)
        oElement = document.getElementsByAttribute('ccode', sLocale.toLowerCase())[0];
    }

    if (oElement) {
      oElement.setAttribute("hidden", false);
      oElement.setAttribute("ui_checked", 'true');
    }

    //Check content locale
    var sCLocale = commonQLS.GetQLSPref("contentlocale");

    oElement = document.getElementById(sCLocale);

    if (!oElement) {
      oElements = document.getElementsByAttribute('ccode', sCLocale.toLowerCase());

      for (i = 0; i < oElements.length; i++) {
        if (oElements[i].nodeName == "menuitem")
          oElement = oElements[i];
      }
    }

    if (!oElement) {
      sCLocale = commonQLS.GetSeamonkeyLocale();
      if (sCLocale && sCLocale != "") {
        oElement = document.getElementById(sCLocale);
        oSBPElement = document.getElementById('sbp_' + sCLocale);
      }

      if (!oElement)
        oElement = document.getElementsByAttribute('ccode', sCLocale.toLowerCase())[0];
    }

    if (oElement) {
      oElement.setAttribute("hidden", false);
      oElement.setAttribute("content_checked", 'true');
    }
/*
    var el = document.getElementById("defaultQLS");
    if (el)
      el.setAttribute('disabled', (sLocale == commonQLS.sDefaultLocale || sCLocale == commonQLS.sDefaultLocale));
    */

    QLSOverlay.QLSInitStatusBar(sLocale, sCLocale);
  },

  UnCheckLocale: function()
  {
    var oElements,
    i;

    function setEl(attr)
    {
      oElements = document.getElementsByAttribute(attr, 'true');
      for (i = 0; i < oElements.length; i++) {
        if (!oElements[i])
          return QLSOverlay.UnCheckLocale();
        oElements[i].setAttribute(attr, 'false');
      }
    }
    setEl('content_checked');
    setEl('ui_checked');

  },


  QLSCheckContentLocale: function(bDontReloadBrowser)
  {
    //if (commonQLS.GetQLSPrefBool("useautoswitch") && !document.getElementById("addressCol2#1") && !document.getElementById("calendar-content-box")) {
    if (!commonQLS.GetQLSPrefBool("useautoswitch") &&
        document.getElementById("addressCol2#1") &&
        document.getElementById("calendar-content-box"))
      return;

    QLSOverlay.QLSInitHostLocales();

    var sFallbackLocale = commonQLS.GetQLSPref("fallbacklocale"),
    sHost = commonQLS.QLSGetHost(window.location.href),
    sLocale = "",
    i;

    for (i = 0; i < commonQLS.QLSHostToLocale.length; i++) {
      if (commonQLS.QLSHostToLocale[i][0] == sHost)
        sLocale = commonQLS.QLSHostToLocale[i][1];
    }
    if (sLocale == "" && typeof(gBrowser) == "object") {
      try {
        gBrowser.selectedBrowser;// jshint ignore:line
      } catch(e) {
        getBrowser();
      }
      var oDocument = gBrowser.selectedBrowser.docShell.QueryInterface(Ci.nsIWebNavigation).document.documentElement;
      if (oDocument) {
        sLocale = oDocument.getAttribute("lang");
        if (!sLocale || sLocale == "")
          sLocale = oDocument.getAttribute("xml:lang");
      }
      if (sLocale && sLocale != "" && sLocale.indexOf('-') == -1)
        sLocale = QLSOverlay.CCodeToLocale(sLocale);
    }
    if ((!sLocale || sLocale == "") && commonQLS.GetQLSPrefBool("autoswitchfortld")) {
      //Check the current Top Level Domain.
      var sTLD = sHost.substr(sHost.lastIndexOf(".") + 1);
      if (sTLD != "") {
        sLocale = commonQLS.QLSTLDToLocale(sTLD);
      }
    }

    if (!sLocale || sLocale == "")
      sLocale = sFallbackLocale;

    if (commonQLS.GetQLSPref("contentlocale") == sLocale)
      return;
    QLSOverlay.UnCheckLocale();

    commonQLS.SetQLSPref("previouscontentlocale", commonQLS.GetQLSPref("contentlocale"));
    commonQLS.SetQLSPref("contentlocale", sLocale);
    QLSOverlay.CheckLocale();

    //var resetAcceptLocale = commonQLS.oQLSPref.getBoolPref("resetaccept");
    //if (resetAcceptLocale)
      QLSOverlay.QLSChangeContentAndDictionaryLocale(sLocale, bDontReloadBrowser);

    setTimeout(function() {
      QLSOverlay.preventAutoSwitchOnDOMContentLoaded = false;
    }, 500);
  },

  QLSModifyAcceptLanguageHeader: function(oHTTPChannel, sURL)
  {
    if (!commonQLS.GetQLSPrefBool("useautoswitch") &&
        commonQLS.GetQLSPrefBool("switch_accept_languages")) //Bool label is reversed... Argh!
      return;

    var sRequestAL = oHTTPChannel.getRequestHeader("Accept-Language");
    if (sRequestAL && sRequestAL == "")
      return;

    QLSOverlay.QLSInitHostLocales();
    var sHost = commonQLS.QLSGetHost(sURL),
    sLocale = "";

    for (var i = 0; i < commonQLS.QLSHostToLocale.length; i++) {
      if (commonQLS.QLSHostToLocale[i][0] == sHost) {
        sLocale = commonQLS.QLSHostToLocale[i][1];
        break;
      }
    }

    if ((!sLocale || sLocale == "") && commonQLS.GetQLSPrefBool("autoswitchfortld")) {
      //Check the current Top Level Domain.
      var sTLD = sHost.substr(sHost.lastIndexOf(".") + 1);
      if (sTLD != "")
        sLocale = commonQLS.QLSTLDToLocale(sTLD);
    }

    if (sLocale && sLocale != "") {
      if (sRequestAL.indexOf(sLocale) < 0 || sRequestAL.indexOf(sLocale) > 7) {
        sRequestAL = sLocale + "," + commonQLS.GetCCode(sLocale) + ";q=1.0," + sRequestAL;
        oHTTPChannel.setRequestHeader("Accept-Language", sRequestAL, false);
      }
    }

  },

  QLSTBCheckAddressField: function(oField)
  {
    if (typeof(awGetInputElement) == "function")
      oField = awGetInputElement(1);
    if (!oField)
      oField = document.getElementById("addressCol2#1");
    if (oField) {
      var sValue = commonQLS.QLSGetEmailAddress(oField.value);
      if (sValue && sValue != "") {
        QLSOverlay.QLSCheckTBContentLocale(sValue);
      }
    }
  },

  QLSCheckTBContentLocale: function(sEmailAddress)
  {
    if (!commonQLS.GetQLSPrefBool("useautoswitch") &&
        !document.getElementById("addressCol2#1"))
      return;

    QLSOverlay.QLSInitHostLocales();
    var sFallbackLocale = commonQLS.GetQLSPref("fallbacklocale"),
    sLocale = "",
    i;

    for (i = 0; i < commonQLS.QLSHostToLocale.length; i++) {
      if (commonQLS.QLSHostToLocale[i][0] == sEmailAddress)
        sLocale = commonQLS.QLSHostToLocale[i][1];
    }

    if ((!sLocale || sLocale == "") && commonQLS.GetQLSPrefBool("autoswitchfortld")) {
      //Check the current Top Level Domain.
      var sTLD = sEmailAddress.substr(sEmailAddress.lastIndexOf(".") + 1);
      if (sTLD != "")
        sLocale = commonQLS.QLSTLDToLocale(sTLD);
    }

    if (!sLocale || sLocale == "")
      sLocale = sFallbackLocale;

    sLocale = sLocale.replace('tb_', '');
    sLocale = sLocale.replace('sbp_', '');
    if (commonQLS.GetQLSPref("contentlocale") == sLocale)
      return;
    /*For TB there's no need to switch if it's not avaiable...*/
    var aMItems = commonQLS.GetQLSPref("visiblemenuitems").split("#"),
    bFound = false;

    for (i = 0; i < aMItems.length; i++) {
      if (sLocale.indexOf(aMItems[i]) >= 0 || aMItems[i].indexOf(sLocale) >= 0)
        bFound = true;
    }

    if (!bFound)
      return;
    QLSOverlay.UnCheckLocale();
    commonQLS.SetQLSPref("previouscontentlocale", commonQLS.GetQLSPref("contentlocale"));
    commonQLS.SetQLSPref("contentlocale", sLocale);
    QLSOverlay.CheckLocale();

    QLSOverlay.QLSChangeContentAndDictionaryLocale(sLocale, false);
  },

  QLSAddHostLocale: function(sLocale)
  {
    QLSOverlay.QLSInitHostLocales();

    var oField = document.getElementById("addressCol2#1"),
    sHost = commonQLS.QLSGetHost(window.location.href),
    i;

    if (oField) {
      for (i = 1; i < 30; i++) {
        oField = document.getElementById("addressCol2#" + i);
        if (oField) {
          sHost = commonQLS.QLSGetEmailAddress(oField.value);
          QLSOverlay.QLSAddHostLocaleCheckAndStore(sLocale, sHost);
        }
      }
    } else
      QLSOverlay.QLSAddHostLocaleCheckAndStore(sLocale, sHost);
  },

  QLSAddHostLocaleCheckAndStore: function(sLocale, sHost)
  {
    var iLen = commonQLS.QLSHostToLocale.length,
    bFound = false;

    for (var i = 0; i < iLen; i++) {
      if (commonQLS.QLSHostToLocale[i][0] == sHost) {
        commonQLS.QLSHostToLocale[i][1] = sLocale;
        bFound = true;
      }
    }
    if (!bFound) {
      commonQLS.QLSHostToLocale[iLen] = [];
      commonQLS.QLSHostToLocale[iLen][0] = sHost;
      commonQLS.QLSHostToLocale[iLen][1] = sLocale;
    }
    QLSOverlay.QLSStoreHostLocalePref();
    QLSOverlay.QLSInitHostLocales(true);
  },

  QLSStoreHostLocalePref: function()
  {
    var aTemp = [],
    sHostLocale = "";

    for (var i = 0; i < commonQLS.QLSHostToLocale.length; i++) {
      aTemp[i] = commonQLS.QLSHostToLocale[i].join(';');
    }
    sHostLocale = aTemp.join(' ');
    commonQLS.SetQLSPref("autoswitchhosts", sHostLocale);
  },

  QLSInitHostLocales: function(bForceReCreate)
  {
    if (bForceReCreate || !commonQLS.QLSHostToLocale) {
      commonQLS.QLSHostToLocale = null;
      commonQLS.QLSHostToLocale = [];

      var aTmp = commonQLS.GetQLSPref("autoswitchhosts").split(' ');

      for (var i = 0; i < aTmp.length; i++) {
        commonQLS.QLSHostToLocale[i] = aTmp[i].split(';');
      }
    }
  },

  QLSInitStatusBar: function(sSelectedLocale, sSelectedCLocale)
  {
    var oSBPanel = document.getElementById('statusbar-QLS'),
    oTBButton = document.getElementById('tbtnCavemanQLS'),
    oSBTooltip = document.getElementById('statusbar-tooltip-QLS-locale'),
    oSBCTooltip = document.getElementById('statusbar-tooltip-QLS-contentlocale'),
    sTooltip,
    sCTooltip;

    if (!oSBPanel && !oSBTooltip && !oSBCTooltip)
      return;
    var sDisplayLocale = sSelectedCLocale;
    if (!sDisplayLocale || sDisplayLocale == "")
      sDisplayLocale = "en-US";
    if (sDisplayLocale.indexOf("-") == -1)
      sDisplayLocale = QLSOverlay.CCodeToLocale(sDisplayLocale);
    oSBPanel.setAttribute('ccode', sDisplayLocale);
    if (oTBButton)
      oTBButton.setAttribute('ccode', sDisplayLocale);
    if (oSBPanel.getAttribute('mode') != commonQLS.GetQLSPref("statusbarmode")) {
      oSBPanel.setAttribute('mode', "text");
      //The timeout is necessary because Ff sometimes renders the icon/text in a diffent order (text first/icon first)
      window.setTimeout(function() {
        QLSOverlay.SetStatusBarMode();
      }, 1);
    }
    oSBPanel.childNodes[0].value = sDisplayLocale;

    var oElement = document.getElementById(sSelectedLocale);
    if (!oElement)
      oElement = document.getElementsByAttribute('ccode', sSelectedLocale.toLowerCase())[0];
    if (!oElement) {
      sSelectedLocale = commonQLS.GetSeamonkeyLocale();
      oElement = document.getElementById(sSelectedLocale);
      if (!oElement)
        oElement = document.getElementsByAttribute('ccode', sSelectedLocale.toLowerCase())[0];
    }
    var oCElement = document.getElementById(sSelectedCLocale);
    if (!oCElement)
      oCElement = document.getElementsByAttribute('ccode', sSelectedCLocale.toLowerCase())[0];
    if (!oCElement) {
      sSelectedCLocale = commonQLS.GetSeamonkeyLocale();
      oCElement = document.getElementById(sSelectedCLocale);
      if (!oCElement)
        oCElement = document.getElementsByAttribute('ccode', sSelectedCLocale.toLowerCase())[0];
    }
    if (!oElement && !oCElement)
      return;
    sTooltip = document.getElementById('statusbar-tooltip-QLS-locale').getAttribute('localizedpart');
    sCTooltip = document.getElementById('statusbar-tooltip-QLS-contentlocale').getAttribute('localizedpart');

    sTooltip = sTooltip.replace("&#37;", "%");
    sCTooltip = sCTooltip.replace("&#37;", "%");
    sTooltip = sTooltip.replace(/%1/, oElement.getAttribute('label'));
    sCTooltip = sCTooltip.replace(/%1/, oCElement.getAttribute('label'));

    oSBTooltip.setAttribute('value', sTooltip);
    oSBCTooltip.setAttribute('value', sCTooltip);
  },

  SetStatusBarMode: function()
  {
    var oSBPanel = document.getElementById('statusbar-QLS');
    if (oSBPanel)
      oSBPanel.setAttribute('mode', commonQLS.GetQLSPref("statusbarmode"));
  },

  SwitchSpellCheckerDictionary: function(sData)
  {
    try {
      var oSC;
      try {
        oSC = Cc['@mozilla.org/spellchecker/engine;1'].getService(Ci.mozISpellCheckingEngine);
      } catch(e) {
        try {
          oSC = Cc['@mozilla.org/spellchecker/myspell;1'].getService(Ci.mozISpellCheckingEngine);
        } catch(e) {
          throw "No Spell checking Service";
        }
      }

      var aDictionaries = {},
      aCount = {},
      aLocales,
      iIndex;

      oSC.getDictionaryList(aDictionaries, aCount);

      aLocales = aDictionaries.value;
      iIndex = commonQLS.ArrayIndexOf(aLocales, sData);
      if (iIndex == -1) {
        //Long lang code does not exist, try short one.
        sData = commonQLS.GetCCode(sData);
        iIndex = commonQLS.ArrayIndexOf(aLocales, sData);
      }

      /*Try to do a "soft" match to find a dictionary that only contains the locale, e.g. fr-classique*/
      if (iIndex == -1) {
        iIndex = commonQLS.ArrayIndexOf(aLocales, sData, true);
        if (iIndex == -1) {
          //Long lang code does not exist, try short one.
          sData = commonQLS.GetCCode(sData);
          iIndex = commonQLS.ArrayIndexOf(aLocales, sData, true);
        }
        if (iIndex >= 0) sData = aLocales[iIndex];
      }
      if (iIndex >= 0) {
        //Only set if the locale is available cause it will break spell checking if you set a not installed dict.
        oSC.dictionary = sData;
        commonQLS.SetSpellCheckerDictionary(sData);
      }

      var oEditor = commonQLS.QLSGetEditorForNode(document.commandDispatcher.focusedElement || document.commandDispatcher.focusedWindow);

      if (typeof(InlineSpellCheckerUI) == "object") {
        InlineSpellCheckerUI.clearSuggestionsFromMenu();
        InlineSpellCheckerUI.clearDictionaryListFromMenu();
      }

      try {
        if (oEditor) {
          if (typeof(InlineSpellCheckerUI) == "object") {
            InlineSpellCheckerUI.init(oEditor);
            InlineSpellCheckerUI.mInlineSpellChecker.spellCheckRange(null);
          } else {
            var oSpellChecker = oEditor.getInlineSpellChecker ? oEditor.getInlineSpellChecker(true) : oEditor.inlineSpellChecker;
            if (oSpellChecker && oSpellChecker.spellChecker) {
              if (iIndex >= 0) oSpellChecker.spellChecker.SetCurrentDictionary(sData);
              oSpellChecker.spellCheckRange(null);
            }
          }
        }
      } catch(e) {}

      //gmail special...
      if (typeof(getBrowser) != "function" || !getBrowser().contentDocument.getElementById("canvas_frame"))
        return;
      var oDocument = getBrowser().contentDocument.getElementById("canvas_frame").contentDocument;
      if (!oDocument)
        return;
      var oElements = oDocument.getElementsByTagName('iframe');
      for (var i = 0; i < oElements.length; i++) {
        if (oElements[i].className && oElements[i].className.indexOf("editable") >= 0 && oElements[i].contentWindow) {
          var oWindow = oElements[i].contentWindow,
          oESession = oWindow.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIWebNavigation).QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIEditingSession);

          if (oESession.windowIsEditable(oWindow))
            oEditor = oESession.getEditorForWindow(oWindow);
          if (oEditor) {
            InlineSpellCheckerUI.init(oEditor);
            InlineSpellCheckerUI.mInlineSpellChecker.spellCheckRange(null);
          }
        }
      }
    } catch(e) {}
  },

  QLSChangeContentAndDictionaryLocale: function(sLocale, bDontReloadBrowser)
  {

    if (!commonQLS.GetQLSPrefBool("switch_contentlocale"))
      return;
    var bReloadBrowser = !bDontReloadBrowser && commonQLS.GetQLSPrefBool("reloadbrowserafterhalswitch");
    if (!commonQLS.GetQLSPrefBool("switch_accept_languages")) //Bool label is reversed... Argh!
      commonQLS.UpdateAcceptLanguages(sLocale, bReloadBrowser,window);
    window.setTimeout(function() {
      QLSOverlay.SwitchSpellCheckerDictionary(sLocale);
    }, 250);
  },

  ChangeLocale: function(sLocale)
  {
    var bChangeLocale = true;

    if (commonQLS.GetQLSPrefBool("switch_alwaysask")) {
      var oSettings = {
        gulocale: commonQLS.GetQLSPrefBool("switch_gulocale"),
        contentlocale: commonQLS.GetQLSPrefBool("switch_contentlocale"),
        alwaysask: commonQLS.GetQLSPrefBool("switch_alwaysask"),
        accepted: false
      };
      window.openDialog("chrome://qls/content/qlsaskswitchsettings.xul", "qls-askswitchsettings", "center", oSettings);
      if (oSettings.accepted) {
        commonQLS.SetQLSPrefBool("switch_gulocale", oSettings.gulocale);
        commonQLS.SetQLSPrefBool("switch_contentlocale", oSettings.contentlocale);
        commonQLS.SetQLSPrefBool("switch_alwaysask", oSettings.alwaysask);
      } else {
        bChangeLocale = false;
        QLSOverlay.CheckLocale();
      }
    }

    if (bChangeLocale) {
      if (commonQLS.GetQLSPrefBool("useautoswitch") &&
          commonQLS.GetQLSPrefBool("switch_contentlocale"))
        QLSOverlay.QLSAddHostLocale(sLocale);

      //website lang
      if (commonQLS.GetQLSPrefBool("switch_contentlocale")) {
        commonQLS.SetQLSPref("previouscontentlocale", commonQLS.GetQLSPref("contentlocale"));
        commonQLS.SetQLSPref("contentlocale", sLocale);

        if (!commonQLS.GetQLSPrefBool("switch_accept_languages")) //Bool label is reversed... Argh!
          commonQLS.UpdateAcceptLanguages(sLocale, commonQLS.GetQLSPrefBool("reloadbrowserafterhalswitch"),window);
        QLSOverlay.SwitchSpellCheckerDictionary(sLocale);
      }

      //gui lang
      if (commonQLS.GetQLSPrefBool("switch_gulocale")) {
        commonQLS.SetQLSPref("previouslocale", commonQLS.GetQLSPref("locale"));
        commonQLS.SetQLSPref("locale", sLocale);
        commonQLS.SetLocale(sLocale);
        commonQLS.SetQLSPref("fallbacklocale", sLocale);

        commonQLS.oALPref.setBoolPref("locale.matchOS", false); //Switch to false since switching would otherwise be pointless..

        if (document.getElementById('taskPopup'))
          commonQLS.ChangeLocaleSeamonkey(sLocale);

        //don't think this is needed anymore...
        commonQLS.DoLocaleWorkAround(QLSOverlay,window);
      }

//might help for some strings
/*
Services.strings.flushBundles();
Services.obs.notifyObservers(null, "chrome-flush-skin-caches", null);
Services.obs.notifyObservers(null, "chrome-flush-caches", null);
*/
      QLSOverlay.CheckLocale();
    }
    //Alert other windows.
    var oWindows = Services.wm.getEnumerator("navigator:browser"),
    oWindow;

    while (oWindows.hasMoreElements()) {
      oWindow = oWindows.getNext();
      if (typeof(oWindow.CheckLocale) == "function")
        oWindow.QLSOverlay.CheckLocale();
    }
  },

  ChangeLocaleBack: function()
  {
    var sLocale = commonQLS.GetQLSPref("previouslocale");
    if (commonQLS.GetQLSPrefBool("switch_contentlocale"))
      sLocale = commonQLS.GetQLSPref("previouscontentlocale");

    QLSOverlay.ChangeLocale(sLocale);
  },

  CCodeToLocale: function(sData)
  {
    var sLocale = "";
    var oElement = document.getElementsByAttribute('ccode', sData)[0];
    if (oElement)
      sLocale = oElement.getAttribute('id');
    sLocale = sLocale.replace('tb_', '');
    sLocale = sLocale.replace('sbp_', '');
    return sLocale;
  },

  showToolsPopup: function(that)
  {
    var popup = document.getElementById('CavemanQLS');
    popup.openPopup(that,'end_before',0,0,false,false);
  }

};

window.addEventListener("load", function load()
{
  window.removeEventListener("load",load,false);
  //let firstPaint go first
  window.setTimeout(function(){
    //QLSOverlay.QLSPreInit();
    QLSOverlay.QLSInit();
  },500);
},false);

window.addEventListener("unload", function load()
{
  window.removeEventListener("unload",load,false);
  QLSOverlay.QLSDeInit();
},false);
